#include <p18f14K50.h>
#include "delays.h"
#include "i2c.h"
#include "eep.h"

#include "ConfigBits14K50.h"
#include "USB_CDC.h"

// ---------------------------------------------------------

unsigned char	USB_QUERY(void);

// ---------------------------------------------------------

#pragma	udata

const unsigned char Stroka1[]={'C','a','l','i','b','r','a','t','e','d',' '};
const unsigned char Stroka2[]={'A','u','t','o',' '};
const unsigned char Stroka3[]={'M','a','n','u','a','l',' '};

unsigned char	USB_BUFF[64];
unsigned char	ByteCount;
signed char		cal_1;
signed char		cal_2;
signed char		mode;
signed int		y;

typedef union combo {
   unsigned int Int;      
   unsigned char Char[2];      
} Tcombo;

Tcombo x;


// =========================================================
// ================   =====================
// =========================================================

void low_isr(void);
void high_isr(void);
			
#pragma code high_vector=0x08
void interrupt_at_high_vector(void)
{
	_asm GOTO high_isr	_endasm
}

#pragma code low_vector=0x18
void interrupt_at_low_vector(void)
{
	_asm GOTO low_isr	_endasm
}

#pragma code /* return to the default code section */

// -----     ----------

#pragma interrupt high_isr

void high_isr (void)
{ 
	USB_CDC_INTERRUPT();
}

// -----     -----------

#pragma interruptlow low_isr

void low_isr (void)
{

}

// ==========================================================
//    LCD
void led_cmd(char lcmd)
{
	Delay100TCYx(0);
	PORTB = 0b00100000;
	PORTC = lcmd;	
	Delay100TCYx(0);
	PORTB = 0b00000000;
	ClrWdt();
}

//     LCD
void lcd_data(char ldata)
{
	Delay100TCYx(0);
	PORTB = 0b10100000;
	PORTC = ldata;	
	Delay100TCYx(0);
	PORTB = 0b10000000;
	ClrWdt();	
}

//  LCD
void led_init()
{
	led_cmd(0b00111000);	//  8 , 2 
	led_cmd(0b00000001);	//  
	led_cmd(0b00000110);	//  .   
	led_cmd(0b00001100);	//   (D=1)
	led_cmd(0b00000001);	//  .    DDRAM
}

//     LCD
void lcd_load()
{
	led_init();

	lcd_data(0xE0);			// ''
	lcd_data(0x20);			// 
	lcd_data(0x20);			// 
	lcd_data(USB_BUFF[0]);	// 
	lcd_data(USB_BUFF[1]);	// 
	lcd_data(USB_BUFF[2]);	// 
	lcd_data(0x2E);			// '.'
	lcd_data(USB_BUFF[4]);	// 

	for(y=0;y<32;y++){
		lcd_data(0x20);
	}

	lcd_data(0xA9);			// '' 
	lcd_data(0x20);			//  
	lcd_data(0x20);			//  
	lcd_data(USB_BUFF[6]);	//  
	lcd_data(USB_BUFF[7]);	//  
	lcd_data(USB_BUFF[8]);	//  
	lcd_data(0x2E);			// '.' 
	lcd_data(USB_BUFF[10]);	// 
}

void read_temp(char tcmd)
{
		StartI2C();
			if(tcmd==0)WriteI2C(0b10011110);	//   ()
			if(tcmd==6)WriteI2C(0b10010110);	//   ()
			WriteI2C(0x00);
		RestartI2C();
			if(tcmd==0)WriteI2C(0b10011111);	//  
			if(tcmd==6)WriteI2C(0b10010111);	//  
			x.Char[1] = ReadI2C();
				AckI2C();
			x.Char[0] = ReadI2C();
				NotAckI2C();
		StopI2C();

		//    
		if(x.Char[1] & (1 << 7)){
			x.Int = (x.Int >> 5);	//    
			x.Int = ~x.Int;			// NOT
			x.Int++;				// +1
			x.Int = (x.Int << 5);	//   
			USB_BUFF[0+tcmd] = '-';
		} else {
			USB_BUFF[0+tcmd] = ' ';
		}
		
		x.Int = (x.Int >> 5);
		if(tcmd==0)x.Int = x.Int * 125 / 100 + cal_1;
		if(tcmd==6)x.Int = x.Int * 125 / 100 + cal_2;
		




		//  
		y = x.Int;
		USB_BUFF[1+tcmd]=0;		
		while (y>=0){
			y-= 100;
			USB_BUFF[1+tcmd]++;
		}

		//  
		y+= 100;
		USB_BUFF[2+tcmd]=0;		
		while (y>=0){
			y-=10;
			USB_BUFF[2+tcmd]++;
		}
		USB_BUFF[1+tcmd] -= 1;
		USB_BUFF[2+tcmd] -= 1;
		USB_BUFF[4+tcmd] = (y+10);
}

void send_temp()
{
		OpenI2C(MASTER, SLEW_ON);
		SSPADD = 0x77;

		read_temp(0);				//   
		read_temp(6);				//   

		CloseI2C();
		TRISC = 0;
		TRISB = 0;

		//USB_BUFF[0] += (0x30);		//  1, , 
		USB_BUFF[1] += (0x30);		//  1, , 
		USB_BUFF[2] += (0x30);		//  1, , 
		USB_BUFF[3] = '.';			//  1, 
		USB_BUFF[4] += (0x30);		//  1, 
		USB_BUFF[5] = ' ';			//   

		//USB_BUFF[6] += (0x30);		//  1, , 
		USB_BUFF[7] += (0x30);		//  2, , 
		USB_BUFF[8] += (0x30);		//  2, , 
		USB_BUFF[9] = '.';			//  2, 
		USB_BUFF[10] += (0x30);		//  2, 
		USB_BUFF[11] = ' ';			//   

		lcd_load();
		WRITE_TO_USB_CDC(& USB_BUFF[0], 12);		//   .
}
void main(void)
{
	// ------------------------------------------------------

	INTCON  = 0x00;				INTCON3 = 0x00;
	PIE1    = 0x00;				PIE2    = 0x00;
	CCP1CON = 0x00;											// .  .
	WPUA	= 0x00;				IOCA	= 0x00;
	WPUB	= 0x00;				IOCB	= 0x00;
	SLRCON	= 0x00;
	SSPCON1 = 0x00;				RCSTA 	= 0x00;				// .  .
	ADCON0  = 0x00;											// . , Default all pins to digital
	ANSEL	= 0x00;				ANSELH	= 0x00;
	TRISA   = 0xFF;				TRISC   = 0xFF;
	TRISB   = 0xFF;
	UCON    = 0x00;
								RCONbits.IPEN     = 1;		//    
	T1CONbits.T1OSCEN = 0x00;	T1CONbits.TMR1CS  = 0;

	// ------------------------------------------------------
	INTCONbits.GIEH   = 1;		INTCONbits.GIEL  =  1;		//  !!!
	// ------------------------------------------------------

	USB_CDC_OPEN();

	cal_1 = Read_b_eep(0x00);
	Busy_eep();
	cal_2 = Read_b_eep(0x01);
	Busy_eep();
	mode  = Read_b_eep(0x02);

	TRISC = 0;
	TRISB = 0;

	led_init();
	// ----------------------------------------------------------
CONTINUE:	ClrWdt();
	// ----------------------------------------------------------

	while(USB_CDC_TX_BUSY());

	ByteCount = READ_FROM_USB_CDC(& USB_BUFF[0]);	//  - ???

	if(ByteCount){
		//   
		if(USB_BUFF[0]=='U'){						
			cal_1++;
		}

		//   
		if(USB_BUFF[0]=='D'){						
			cal_1--;
		}

		//   
		if(USB_BUFF[0]=='u'){						
			cal_2++;
		}

		//   
		if(USB_BUFF[0]=='d'){						
			cal_2--;
		}

		//   
		//   "Calibrated"  USB
		if(USB_BUFF[0]=='S'){						
			Write_b_eep(0x00, cal_1);
			Busy_eep();
			Write_b_eep(0x01, cal_2);
			Busy_eep();
			WRITE_TO_USB_CDC(& Stroka1[0], 11);		
 		}

		//    
		if((USB_BUFF[0]=='R')&&(mode==0x01)){		
			send_temp();
 		}

		//    
		if(USB_BUFF[0]=='A'){						
			mode=0x00;
			Write_b_eep(0x02, mode);				
			Busy_eep();
			WRITE_TO_USB_CDC(& Stroka2[0], 5);		//   "Auto"  USB
 		}
		if(USB_BUFF[0]=='M'){						//    
			mode=0x01;
			Write_b_eep(0x02, mode);				
			Busy_eep();
			WRITE_TO_USB_CDC(& Stroka3[0], 7);		//   "Manual"  USB
 		}
		
	}


	//   
	//  , ,  ,  4   .
	if(mode==0x00){
		send_temp();

		//   ( 4  +    )
		y=90;
		while(y>0){
			Delay1KTCYx(0);
			ClrWdt();
			y--;
		}
	}

		
goto CONTINUE;
}
